@@ -1,9 +1,14 @@
+import json
 from datetime import datetime
 from enum import Enum
+from pathlib import Path
 from typing import Iterator, Optional
 from urllib.parse import urljoin
 
 import requests
+from jsonschema import Draft7Validator
+
+SCHEMA_DIR = Path(__file__).parent / "schemas"
 
 
 class Constants(str, Enum):
@@ -11,6 +16,46 @@ class Constants(str, Enum):
     def values(cls):
         return tuple(m.value for m in cls)
 
+    def __str__(self) -> str:
+        return str(self.value)
+
+
+class CveRecordValidationError(Exception):
+    def __init__(self, message, errors):
+        super().__init__(message)
+        self.errors = errors
+
+
+class CveRecord:
+    class Schemas(Constants):
+        CNA_PUBLISHED = next(SCHEMA_DIR.glob("published_cna_container_*.json"))
+        CNA_REJECTED = next(SCHEMA_DIR.glob("rejected_cna_container_*.json"))
+        ADP = next(SCHEMA_DIR.glob("adp_container_*.json"))
+        V5_SCHEMA = next(SCHEMA_DIR.glob("CVE_JSON_5.0_bundled_*.json"))
+
+    @classmethod
+    def validate(cls, cve_json: dict, schema_path: Optional[str] = None) -> None:
+        """Validate a CVE record against a JSON schema.
+
+        Optionally, specify a path to a JSON schema file with which to validate the record; if not
+        specified, the Published CNA container schema bundled in cvelib/schemas/ is used. All
+        other schemas in that directory must be explicitly specified, e.g.:
+
+        CveRecord.validate(cve_json, schema_path=CveRecord.Schemas.CNA_REJECTED)
+        """
+        if schema_path is None:
+            schema_path = cls.Schemas.CNA_PUBLISHED
+
+        with open(schema_path) as schema_file:
+            schema = json.load(schema_file)
+
+        validator = Draft7Validator(schema)
+        errors = [error.message for error in sorted(validator.iter_errors(cve_json), key=str)]
+        if errors:
+            raise CveRecordValidationError(
+                f"Schema validation against {schema_path} failed", errors
+            )
+
 
 class CveApi:
     ENVS = {
@@ -87,29 +132,72 @@ class CveApi:
     def _put(self, path: str, **kwargs) -> requests.Response:
         return self._http_request("put", path, **kwargs)
 
-    def publish(self, cve_id: str, cve_json: dict) -> dict:
+    @staticmethod
+    def _extract_cna_container(cve_json: dict) -> dict:
+        """Check if we received a full v5 record and extract just the CNA container part from it.
+
+        A record "looks" like a full one if it has a dataType=CVE_RECORD attribute. Otherwise,
+        it is assumed to be the CNA container.
+        """
+        if cve_json.get("dataType", "") == "CVE_RECORD":
+            return cve_json["containers"]["cna"]
+        return cve_json
+
+    def _add_provider_metadata(self, cve_json: dict) -> dict:
+        """Add the providerMetadata objects to a CNA container if one is not present.
+
+        The orgId is the only required element of the providerMetadata object, and we can fetch
+        it from the API using the org short name provided by the user (when this class is
+        instantiated.)
+        """
+        if "providerMetadata" not in cve_json:
+            org_id = self.show_org()["UUID"]
+            cve_json["providerMetadata"] = {"orgId": org_id}
+        return cve_json
+
+    def publish(self, cve_id: str, cve_json: dict, validate: bool = True) -> dict:
         """Publish a CVE from a JSON object representing the CNA container data."""
+        cve_json = self._extract_cna_container(cve_json)
+        cve_json = self._add_provider_metadata(cve_json)
+        if validate:
+            CveRecord.validate(cve_json, CveRecord.Schemas.CNA_PUBLISHED)
+
         cve_json = {"cnaContainer": cve_json}
         response = self._post(f"cve/{cve_id}/cna", json=cve_json)
         response.raise_for_status()
         return response.json()
 
-    def update_published(self, cve_id: str, cve_json: dict) -> dict:
+    def update_published(self, cve_id: str, cve_json: dict, validate: bool = True) -> dict:
         """Update a published CVE record from a JSON object representing the CNA container data."""
+        cve_json = self._extract_cna_container(cve_json)
+        cve_json = self._add_provider_metadata(cve_json)
+        if validate:
+            CveRecord.validate(cve_json, CveRecord.Schemas.CNA_PUBLISHED)
+
         cve_json = {"cnaContainer": cve_json}
         response = self._put(f"cve/{cve_id}/cna", json=cve_json)
         response.raise_for_status()
         return response.json()
 
-    def reject(self, cve_id: str, cve_json: dict) -> dict:
+    def reject(self, cve_id: str, cve_json: dict, validate: bool = True) -> dict:
         """Reject a CVE from a JSON object representing the CNA container data."""
+        cve_json = self._extract_cna_container(cve_json)
+        cve_json = self._add_provider_metadata(cve_json)
+        if validate:
+            CveRecord.validate(cve_json, CveRecord.Schemas.CNA_REJECTED)
+
         cve_json = {"cnaContainer": cve_json}
         response = self._post(f"cve/{cve_id}/reject", json=cve_json)
         response.raise_for_status()
         return response.json()
 
-    def update_rejected(self, cve_id: str, cve_json: dict) -> dict:
+    def update_rejected(self, cve_id: str, cve_json: dict, validate: bool = True) -> dict:
         """Update a rejected CVE record from a JSON object representing the CNA container data."""
+        cve_json = self._extract_cna_container(cve_json)
+        cve_json = self._add_provider_metadata(cve_json)
+        if validate:
+            CveRecord.validate(cve_json, CveRecord.Schemas.CNA_REJECTED)
+
         cve_json = {"cnaContainer": cve_json}
         response = self._put(f"cve/{cve_id}/reject", json=cve_json)
         response.raise_for_status()
@@ -158,10 +246,10 @@ class CveApi:
 
     def list_cves(
         self,
-        year: str = None,
-        state: str = None,
-        reserved_lt: datetime = None,
-        reserved_gt: datetime = None,
+        year: Optional[str] = None,
+        state: Optional[str] = None,
+        reserved_lt: Optional[datetime] = None,
+        reserved_gt: Optional[datetime] = None,
     ) -> Iterator[dict]:
         params = {}
         if year:
